
/* ygtcpip.c
   (C) 2011,2017,2020 Yann Guidon whygee@f-cpu.org
   The source code of this project is released under the AGPLv3 or later,
   see https://www.gnu.org/licenses/agpl-3.0.en.html

version 20200415: split from HTTaP.h, removed IPv6,
  added some portability and simplified some parts
 */

const int const_one = 1, const_zero=0; /* required by setsockopt() and others */

//////////////////////////////////////////////////////////////////////////

void net_exit(char *msg) {
  fflush(NULL);
  net_perror(msg);
  exit(EXIT_FAILURE);
}

SOCKET_TYPE create_IPv4_socket(  // return the socket number, or -1 if error
    char *ip,                    // just a string, NULL if server
    char *port,                  // another ASCIIZ string
    struct addrinfo *created_ai, // where to save the parameters of the socket
    struct addrinfo *hints) {    // hints from the caller, to be filled right before calling here

  int flags;
  SOCKET_TYPE sock;
  struct addrinfo *list_adresses, *p_adress;

  hints->ai_family = AF_INET; // IPv4

  if ((flags=getaddrinfo(ip, port, hints, &list_adresses)) != 0) {
    printf("\ngetaddrinfo: %s\n", gai_strerror(flags));
    return SOCKET_ERROR;
  }

  sock=SOCKET_ERROR;
  p_adress = list_adresses;

  /* scan the linked list of addresses and tries to open sockets */
  while (p_adress) {
    if ((sock = socket(p_adress->ai_family,
        p_adress->ai_socktype, p_adress->ai_protocol)) != INVALID_SOCKET) {
      if (created_ai) // save the socket info if a destination pointer is provided
        memcpy(created_ai, p_adress, sizeof(struct addrinfo));
      break;
    }
    p_adress=p_adress->ai_next; /* proceed to the next entry */
  }

  if (sock == SOCKET_ERROR)
    printf("create_IPv4_socket : No suitable network socket found\n");
  freeaddrinfo(list_adresses); //Never forget to free the memory used by the linked list

  return sock;
}
